<?php

namespace Drupal\bing_indexing_api\Service;

use Drupal\Core\State\StateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\ClientInterface;

/**
 * Service class for calling the Bing Indexing API.
 *
 * @ingroup bing_index_api
 */
class BingIndexApi {

  /**
   * The Google Index API End Point.
   */
  const END_POINT = 'https://www.bing.com/webmaster/api.svc/json/SubmitUrlbatch?apikey=';

  /**
   * Drupal\Core\State\StateInterface definition.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The GuzzleHttp Client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * Callback Controller constructor.
   *
   * @param \Drupal\Core\State\StateInterface $state
   *   The State Service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The Logging Service.
   * @param \GuzzleHttp\ClientInterface $client
   *   Http Client.
   */
  public function __construct(StateInterface $state, LoggerChannelFactoryInterface $loggerFactory, ClientInterface $client) {
    $this->state = $state;
    $this->loggerFactory = $loggerFactory->get('Bing Index API');
    $this->client = $client;
  }

  /**
   * Update url in the Bing Index API.
   *
   * @param string $url
   *   The url we are updating.
   */
  public function updateUrl($url) {
    $this->callApi($url);
  }

  /**
   * Mechanism to call the API.
   *
   * @param string $url
   *   The url we are calling.
   */
  protected function callApi($url) {
    $domain = $this->state->get('bing_index_api_base_domain');
    $apiKey = $this->state->get('bing_index_api_key');

    if (!$domain || !$apiKey) {
      $this->loggerFactory->error('Please check Bing Api settings: domain and key');
      return;
    }

    $response = $this->client->post(self::END_POINT . $apiKey, [
      'json' => [
        'siteUrl' => $domain,
        'urlList' => [$domain . $url],
      ],
      'headers' => [
        'Content-type' => 'application/json; charset=utf-8',
      ],
    ]);

    $status_code = $response->getStatusCode();

    // Let the peoples know what happened.
    switch ($status_code) {
      case '200':
        $this->loggerFactory->notice('Successfully updated %url', [
          '%url' => $url,
        ]);
        break;

      default:
        $body = json_decode($response->getBody()->getContents(), TRUE);
        $msg = $body["error"]["message"] ?? 'Bing Index API Returned a Status code of ' . $status_code;
        $this->loggerFactory->error($msg);
        break;
    }
  }

}
